==> server-core-3.0.3-sources.jar: com/xebialabs/deployit/security/permission/PermissionHelper.java <==
package com.xebialabs.deployit.security.permission;

import com.xebialabs.deployit.security.SecurityServiceLocator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

public class PermissionHelper {
	static void grantPermission(Permission permission, String user, String onConfigurationItem) {
		grantPermission(constructPermissionName(permission, onConfigurationItem), user);
	}

	static void grantPermission(Permission permission, String user) {
		grantPermission(permission.getPermissionName(), user);
	}

	private static void grantPermission(String permission, String user) {
		SecurityServiceLocator.getPermissionService().grantPermission(permission, user);
	}

	static void revokePermission(Permission permission, String user, String onConfigurationItem) {
		revokePermission(constructPermissionName(permission, onConfigurationItem), user);
	}

	static void revokePermission(Permission permission, String user) {
		revokePermission(permission.getPermissionName(), user);
	}

	private static void revokePermission(String permission, String user) {
		SecurityServiceLocator.getPermissionService().denyPermission(permission, user);
	}

	static boolean hasPermission(Permission permission) {
		return hasPermission(permission.getPermissionName());
	}

	private static boolean hasPermission(String permission) {
		return SecurityServiceLocator.getPermissionService().hasPermission(permission);
	}

	static boolean hasPermission(Permission permission, List<String> onConfigurationItems) {
		boolean allowed = !onConfigurationItems.isEmpty();
		for (String onConfigurationItem : onConfigurationItems) {
			allowed = allowed && hasPermission(permission, onConfigurationItem);
		}
		return allowed || hasPermission(permission);
	}

	private static boolean hasPermission(Permission permission, String onConfigurationItem) {
		return hasPermission(constructPermissionName(permission, onConfigurationItem));
	}

	static boolean hasPermission(Permission permission, String principal, String onConfigurationItem) {
		return SecurityServiceLocator.getPermissionService().hasPermission(constructPermissionName(permission, onConfigurationItem), principal);
	}

	private static String constructPermissionName(Permission permission, String onConfigurationItem) {
		final StringBuilder builder = new StringBuilder(permission.getPermissionName());
		if (onConfigurationItem != null) {
			logger.info("constructing permission: {} with {} ", builder.toString(), onConfigurationItem);
			builder.append("_").append(onConfigurationItem.replaceAll("/", "\\$"));
		}
		return builder.toString();
	}

	private static final Logger logger = LoggerFactory.getLogger(PermissionHelper.class);
}

==> server-core-3.5.0-sources.jar: com/xebialabs/deployit/security/permission/PermissionHelper.java <==
package com.xebialabs.deployit.security.permission;

import static com.google.common.collect.Lists.newArrayList;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.xebialabs.deployit.security.SecurityServiceLocator;

public class PermissionHelper {
	static void grantPermission(Permission permission, String user, String onConfigurationItem) {
		grantPermission(constructPermissionName(permission, onConfigurationItem), user);
	}

	static void grantPermission(Permission permission, String user) {
		grantPermission(permission.getPermissionName(), user);
	}

	private static void grantPermission(String permission, String user) {
		SecurityServiceLocator.getPermissionService().grantPermission(permission, user);
	}

	static void revokePermission(Permission permission, String user, String onConfigurationItem) {
		revokePermission(constructPermissionName(permission, onConfigurationItem), user);
	}

	static void revokePermission(Permission permission, String user) {
		revokePermission(permission.getPermissionName(), user);
	}

	private static void revokePermission(String permission, String user) {
		SecurityServiceLocator.getPermissionService().denyPermission(permission, user);
	}

	static boolean hasPermission(Permission permission) {
		return hasPermission(permission.getPermissionName());
	}

	private static boolean hasPermission(String permission) {
		return SecurityServiceLocator.getPermissionService().hasLoggedInUserPermission(permission);
	}

	static boolean hasPermission(Permission permission, List<String> onConfigurationItems) {
		boolean allowed = !onConfigurationItems.isEmpty();
		for (String onConfigurationItem : onConfigurationItems) {
            List<String> fullTree = getFullTreeAsSeparateNodes(onConfigurationItem);
            boolean hasPermissionOnTreePart = false;
            for (String s : fullTree) {
                hasPermissionOnTreePart = hasPermissionOnTreePart || hasPermission(permission, s);
            }
            allowed = allowed && hasPermissionOnTreePart;
		}
		return allowed || hasPermission(permission);
	}

    private static List<String> getFullTreeAsSeparateNodes(String onConfigurationItem) {
        ArrayList<String> list = newArrayList();
        list.add(onConfigurationItem);
        for (int index = onConfigurationItem.indexOf('/'); index > -1; index = onConfigurationItem.indexOf('/', index + 1)) {
            list.add(onConfigurationItem.substring(0, index));
        }

        logger.debug("Split off tree: {} to {}", onConfigurationItem, list);

        return list;
    }

    private static boolean hasPermission(Permission permission, String onConfigurationItem) {
		return hasPermission(constructPermissionName(permission, onConfigurationItem));
	}

	static boolean hasPermission(Permission permission, String principal, String onConfigurationItem) {
		return SecurityServiceLocator.getPermissionService().hasUserPermission(constructPermissionName(permission, onConfigurationItem), principal);
	}

	private static String constructPermissionName(Permission permission, String onConfigurationItem) {
		final StringBuilder builder = new StringBuilder(permission.getPermissionName());
		if (onConfigurationItem != null) {
			logger.debug("constructing permission: {} with {} ", builder.toString(), onConfigurationItem);
			builder.append("_").append(onConfigurationItem.replaceAll("/", "\\$"));
		}
		return builder.toString();
	}

	private static final Logger logger = LoggerFactory.getLogger(PermissionHelper.class);
}
